#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Scrollbar.h>
#include <X11/Xaw/Repeater.h>
#include <Mowitz/Mowitz.h>
#include "MwHtml.h"


#define MENUBAR (1)
#define NAVBAR (2)
#define PERSBAR (4)
#define COMPBAR (8)

static int bars = MENUBAR;

static Widget topLevel, topbox;
static Widget html;
static Widget menubar, menubox, menubutton, menu;
static Widget navbar, navframe, navbox;
static Widget persbar, persframe, persbox, loc_label, loc_text;
static Widget statbar, label1, label2, compbar, compframe, compbox;
static Widget tooltip;
static Widget viewport, vscroll, hscroll, tab, tabl, tabr;

static String fallback[] = {
#include "kylie-ad.h"
	NULL
};

static char *myhtml =
"<html>
<head>
</head>
<body>
<h1>Header 1</h1>
Det hr r ett enkelt testdokument fr min HTML-parser.
Det hr br komma p samma rad.
<pre>
int <i>main</i>(void)
{
        return <b>0</b>;
}
</pre>
<ul>
<li>foo
<li>r&auml;ksm&ouml;rg&aring;sar
<li>baz
</ul>

Slut p listan!

<h2>Header 2</h2>
Hr kommer en ordnad lista.
<ol>
<li>foo
<ul>
<li>cro
<li>mag
<li>non
</ul>
<li>bar<b>bold</b>normal
<ol>
<li>cro
<li>mag
<li>non
<li>And finally a longer item to test line wrap
</ol>
<li>baz
</ol>

Det var vl inte s svrt? Nsta vecka skall vi lra oss att kopiera
en fil!
<p>
Det hr r ett nytt stycke.
<table>
<tr>
<th>En cell.</th>
<th>En cell.</th>
<th>En cell.</th>
</tr>
<tr>
<td>En cell till.
<td>En cell till.
<td>En cell till.
</tr>
<tr>
<td>En cell till.</td>
<td>En cell till.</td>
<td>En cell till.</td>
</tr>
</table>
<br>
Radbrytning.
<p>
Och nytt stycke igen.
</body>
</html>";

static void cb_open(Widget w, XtPointer client_data, XtPointer call_data)
{
	char path[1024] = ".";
	char name[1024] = "";
	char *patterns[] = { "All files (*)", NULL };
	char fmt[1024] = "";
	char *extra = NULL;
	int ext = 0;

	MwFileselInput(topLevel, path, name, patterns, fmt, extra, ext);
}

static void cb_quit(Widget w, XtPointer client_data, XtPointer call_data)
{
	MwMallocExit();
	exit(0);
}

static void cb_error(Widget w, XtPointer client_data, XtPointer call_data)
{
	MwErrorBox(topLevel, "Nothing to see here.");
}


static void cb_alert(Widget w, XtPointer client_data, XtPointer call_data)
{
	char *buttons[] = {"Abort", "Retry", "Continue", "Fail"};
	MwAlertBox(topLevel, "An unexpected error didn't occur at all.",
		buttons, 4);
}


static void cb_about(Widget w, XtPointer client_data, XtPointer call_data)
{
printf("cb_about(%s)\n", XtName(w));
	MwAboutSiag(topLevel);
}

static void cb_click(Widget w, XtPointer client_data, XtPointer call_data)
{
	char *u = (char *)call_data;
printf("cb_click(%s)\n", u);
	XtVaSetValues(html, XtNurl, u, (char *)0);
}

static void cb_url(Widget w, XtPointer client_data, XtPointer call_data)
{
	char *u = (char *)call_data;
printf("cb_url(%s)\n", u);
#if 1
	MwComboTextChange(loc_text, u);
#endif
}

static void cb_loc(Widget w, XtPointer client_data, XtPointer call_data)
{
	char *u = (char *)call_data;
	XtVaSetValues(html, XtNurl, u, (char *)0);
}

static void quit(Widget w, XEvent *event, String *params, Cardinal *n)
{
	exit(0);
}

static void vscroll_jump(Widget w, XtPointer client_data, XtPointer call_data)
{
	float top;
	int top_row, th;

	th = 20*100;
	XtVaGetValues(w, XtNtopOfThumb, &top, (char *)0);
	top_row = top*th;
	XtVaSetValues(html, XtNtopRow, top_row, (char *)0);
}

static void vscroll_scroll(Widget w, XtPointer client_data, XtPointer call_data)
{
	int i = (long)call_data;
	Dimension length, height;
	int top_row, th;

	XtVaGetValues(w, XtNlength, &length, (char *)0);
	XtVaGetValues(html,
		XtNheight, &height,
		XtNtopRow, &top_row,
		(char *)0);
	th = 20*100;
	if (i < 0) {
		if ((length / -i) > 15) {
			top_row -= 0.1*height;
		} else {
			top_row -= 0.9*height;
		}
	} else {
		if ((length / i) > 15) {
			top_row += 0.1*height;
		} else {
			top_row += 0.9*height;
		}
	}
	if (top_row > th) top_row = th;
	if (top_row < 0) top_row = 0;
	XtVaSetValues(html, XtNtopRow, top_row, (char *)0);
	XawScrollbarSetThumb(w, (double)top_row/th, 0.0);
}

static void hscroll_jump(Widget w, XtPointer client_data, XtPointer call_data)
{
        float top;
        int top_col, pw;

        pw = 1000;
        XtVaGetValues(w, XtNtopOfThumb, &top, (char *)0);
        top_col = top*pw;
        XtVaSetValues(html,
                XtNtopCol, top_col,
                (char *)0);
}

static void hscroll_scroll(Widget w, XtPointer client_data, XtPointer call_data)
{
        int i = (long) call_data;
        Dimension length, width;
        int top_col, change, pw;

        pw = 1000;
        XtVaGetValues(w, XtNlength, &length, (char *)0);
        XtVaGetValues(html,
                        XtNwidth, &width,
                        XtNrichtextTopCol, &top_col,
                        (char *)0);
        if (i < 0) {
                if ((length / -i) > 15) {
                        change = -0.1*width;
                } else {
                        change = -0.9*width;
                }
        } else {
                if ((length / i) > 15) {
                        change = 0.1*width;
                } else {
                        change = 0.9*width;
                }
        }
        top_col += change;
        if (top_col > pw) top_col = pw;
        if (top_col < 0) top_col = 0;
        XtVaSetValues(html,
                XtNrichtextTopCol, top_col,
                (char *)0);
        XawScrollbarSetThumb(w, (double)top_col/pw, 0.0);
}

static XtActionsRec actions[] =
{
	{"quit", quit}
};

static void remake_ylayout(void)
{
	char b[100];
	sprintf(b, "%s %s %s 100%% 30",
		(bars&MENUBAR)?"30":"0",
		(bars&NAVBAR)?"30":"0",
		(bars&PERSBAR)?"30":"0");
	XtVaSetValues(topbox,
		XtNyLayout, b,
		(char *)0);
	sprintf(b, "4 50 4 100%% 4 %s",
		(bars&COMPBAR)?"175 4":"0 0");
	XtVaSetValues(statbar,
		XtNxLayout, b,
		(char *)0);
}

static void attach(Widget w, XtPointer client_data, XtPointer call_data)
{
	Widget vw = (Widget)call_data;
printf("attach(%s), vw is %s, parent is %s\n", XtName(w), XtName(vw), XtName(XtParent(w)));
	if (vw == menubar) bars |= MENUBAR;
	else if (vw == navbar) bars |= NAVBAR;
	else if (vw == persbar) bars |= PERSBAR;
	else if (vw == compbar) bars |= COMPBAR;
	remake_ylayout();
}

static void detach(Widget w, XtPointer client_data, XtPointer call_data)
{
	Widget vw = (Widget)call_data;
printf("detach(%s)\n", XtName(w));
	if (vw == menubar) bars &= ~MENUBAR;
	else if (vw == navbar) bars &= ~NAVBAR;
	else if (vw == persbar) bars &= ~PERSBAR;
	else if (vw == compbar) bars &= ~COMPBAR;
	remake_ylayout();
}

static Widget add_command(Widget pw, void (*cb)(Widget, XtPointer, XtPointer),
		XtPointer closure, char *pm, char *t)
{
	Widget w;
	Pixmap pm_return;
	Pixel color;
printf("add_command(%s, %s)\n", pm, t);
	XtVaGetValues(pw, XtNbackground, &color, (char *)0);
	w = XtVaCreateManagedWidget("tb_cmd",
		commandWidgetClass, pw,
		XtNforeground, color,
		(char *)0);
	pm_return = MwLoadPixmap(XtDisplay(pw), color, pm);
	XtVaSetValues(w, XtNbitmap, pm_return, (char *)0);
	XtAddCallback(w, XtNcallback, cb, closure);
	MwTooltipAdd(tooltip, w, _(t));
	return w;
}

int main(int argc, char **argv)
{
	Atom wm_protocols, wm_delete_window;
	XtAppContext ac;
	Widget w;
	char b[1024];

	if (getenv("PIXPATH") == NULL) {
		sprintf(b, "PIXPATH=%s", DEFAULT_PIXPATH);
		putenv(b);
	}
	topLevel = XtVaAppInitialize(&ac, "Kylie",
		NULL, 0,
		&argc, argv,
		fallback,
		XtNwidth, 300,
		XtNheight, 200,
		(char *)0);
	XtAppAddActions(ac, actions, XtNumber(actions));
	tooltip = XtVaCreatePopupShell("tooltip",
		mwTooltipWidgetClass, topLevel,
		(char *)0);
	MwHighlightInit(topLevel);
	topbox = XtVaCreateManagedWidget("topbox",
		mwRudegridWidgetClass, topLevel,
		(char *)0);
	menubar = XtVaCreateManagedWidget("menubar",
		mwRudegridWidgetClass, topbox,
		(char *)0);
	MwMakeHandle(menubar, menubar, detach, attach);
	menubox = XtVaCreateManagedWidget("menubox",
		mwMenuBarWidgetClass, menubar,
		(char *)0);
	bars |= MENUBAR;
	menubutton = XtVaCreateManagedWidget("menubutton",
		mwMBButtonObjectClass, menubox,
		XtNlabel, "File",
		XtNmenu_name, "filemenu",
		(char *)0);
	menu = XtVaCreatePopupShell("filemenu",
		mwMenuWidgetClass, menubox, NULL, 0);
	w = XtVaCreateManagedWidget("Open",
		mwLabelMEObjectClass, menu,
		XtNlabel, "Open",
		(char *)0);
	XtAddCallback(w, XtNcallback, cb_open, NULL);
	XtVaCreateManagedWidget("-",
		mwLineMEObjectClass, menu,
		(char *)0);
	w = XtVaCreateManagedWidget("Exit",
		mwLabelMEObjectClass, menu,
		XtNlabel, "Exit",
		(char *)0);
	XtAddCallback(w, XtNcallback, cb_quit, NULL);

	menubutton = XtVaCreateManagedWidget("menubutton",
		mwMBButtonObjectClass, menubox,
		XtNlabel, "Misc",
		XtNmenu_name, "miscmenu",
		(char *)0);
	menu = XtVaCreatePopupShell("miscmenu",
		mwMenuWidgetClass, menubox, NULL, 0);
	w = XtVaCreateManagedWidget("Alert",
		mwLabelMEObjectClass, menu,
		XtNlabel, "Alert",
		(char *)0);
	XtAddCallback(w, XtNcallback, cb_alert, NULL);
	w = XtVaCreateManagedWidget("Error",
		mwLabelMEObjectClass, menu,
		XtNlabel, "Error",
		(char *)0);
	XtAddCallback(w, XtNcallback, cb_error, NULL);

	menubutton = XtVaCreateManagedWidget("menubutton",
		mwMBButtonObjectClass, menubox,
		XtNlabel, "Help",
		XtNmenu_name, "helpmenu",
		(char *)0);
	menu = XtVaCreatePopupShell("helpmenu",
		mwMenuWidgetClass, menubox, NULL, 0);
	XtVaCreateManagedWidget("Contents",
		mwLabelMEObjectClass, menu,
		XtNlabel, "Contents",
		(char *)0);
	XtVaCreateManagedWidget("-",
		mwLineMEObjectClass, menu,
		(char *)0);
	w = XtVaCreateManagedWidget("About",
		mwLabelMEObjectClass, menu,
		XtNlabel, "About",
		(char *)0);
	XtAddCallback(w, XtNcallback, cb_about, NULL);

	navbar = XtVaCreateManagedWidget("navbar",
		mwRudegridWidgetClass, topbox,
		(char *)0);
	bars |= NAVBAR;
	MwMakeHandle(navbar, navbar, detach, attach);
	navframe = XtVaCreateManagedWidget("navframe",
		mwFrameWidgetClass, navbar,
		(char *)0);
	navbox = XtVaCreateManagedWidget("navbox",
		boxWidgetClass, navframe,
		(char *)0);
	add_command(navbox, cb_error, NULL, "back.xpm", "Back");
	add_command(navbox, cb_error, NULL, "forward.xpm", "Forward");
	add_command(navbox, cb_error, NULL, "reload.xpm", "Reload");
	add_command(navbox, cb_error, NULL, "cancel.xpm", "Cancel");
	add_command(navbox, cb_error, NULL, "home.xpm", "Home");
	add_command(navbox, cb_error, NULL, "fld_open.xpm", "Open");
	add_command(navbox, cb_error, NULL, "save.xpm", "Save");

	persbar = XtVaCreateManagedWidget("persbar",
		mwRudegridWidgetClass, topbox, (char *)0);
	bars |= PERSBAR;
	MwMakeHandle(persbar, persbar, detach, attach);
	persframe = XtVaCreateManagedWidget("persframe",
		mwFrameWidgetClass, persbar,
		(char *)0);
	persbox = XtVaCreateManagedWidget("persbox",
		mwRudegridWidgetClass, persframe,
		(char *)0);
	loc_label = XtVaCreateManagedWidget("loc_label",
		labelWidgetClass, persbox,
		XtNlabel, _("Location:"),
		(char *)0);
	loc_text = XtVaCreateManagedWidget("loc_text",
		mwComboWidgetClass, persbox,
		(char *)0);
	XtVaSetValues(loc_text, "The textfield is borken");
	XtAddCallback(loc_text, XtNtextCallback, cb_loc, NULL);
	XtAddCallback(loc_text, XtNlistCallback, cb_loc, NULL);

	statbar = XtVaCreateManagedWidget("statbar",
		mwRudegridWidgetClass, topbox, (char *)0);
	label1 = XtVaCreateManagedWidget("label1",
		labelWidgetClass, statbar,
		(char *)0);
	label2 = XtVaCreateManagedWidget("label2",
		labelWidgetClass, statbar,
		(char *)0);

	compbar = XtVaCreateManagedWidget("compbar",
		mwRudegridWidgetClass, statbar, (char *)0);
	bars |= COMPBAR;
	MwMakeHandle(compbar, compbar, detach, attach);
	compframe = XtVaCreateManagedWidget("compframe",
		mwFrameWidgetClass, compbar,
		(char *)0);
	compbox = XtVaCreateManagedWidget("compbox",
		boxWidgetClass, compframe,
		(char *)0);
	add_command(compbox, cb_error, NULL, "navigator.xpm", "Navigator");
	add_command(compbox, cb_error, NULL, "mail.xpm", "Mail");
	add_command(compbox, cb_error, NULL, "news.xpm", "News");
	add_command(compbox, cb_error, NULL, "addressbook.xpm", "Address book");
	add_command(compbox, cb_error, NULL, "composer.xpm", "Editor");

	viewport = XtVaCreateManagedWidget("viewport",
		mwRudegridWidgetClass, topbox,
		(char *)0);

	html = XtVaCreateManagedWidget("html",
		mwHtmlWidgetClass, viewport,
		XtNhtml, myhtml,
		XtNstatus, label2,
		(char *)0);
	XtAddCallback(html, XtNcallback, cb_click, NULL);
	XtAddCallback(html, XtNchangeUrl, cb_url, NULL);
	if (argc < 2) {
		XtVaSetValues(html, XtNhtml, myhtml, (char *)0);
	} else {
		XtVaSetValues(html, XtNurl, argv[1], (char *)0);
	}
	vscroll = XtVaCreateManagedWidget("vscroll",
		scrollbarWidgetClass, viewport,
		(char *)0);
	XtAddCallback(vscroll, XtNjumpProc, vscroll_jump, NULL);
	XtAddCallback(vscroll, XtNscrollProc, vscroll_scroll, NULL);
	hscroll = XtVaCreateManagedWidget("hscroll",
		scrollbarWidgetClass, viewport,
		(char *)0);
	XtAddCallback(hscroll, XtNjumpProc, hscroll_jump, NULL);
	XtAddCallback(hscroll, XtNscrollProc, hscroll_scroll, NULL);
	tab = XtVaCreateManagedWidget("tab",
		mwTabbingWidgetClass, viewport,
		(char *)0);
	MwTabbingInsert(tab, "Page 1", -1);
	MwTabbingInsert(tab, "Page 2", -1);
	tabl = XtVaCreateManagedWidget("tabl",
		repeaterWidgetClass, viewport,
		(char *)0);
	tabr = XtVaCreateManagedWidget("tabr",
		repeaterWidgetClass, viewport,
		(char *)0);

	tooltip = XtVaCreatePopupShell("tooltip",
		mwTooltipWidgetClass, topLevel,
		(char *)0);
	XtRealizeWidget(topLevel);
	wm_protocols = XInternAtom(XtDisplay(topLevel),
				"WM_PROTOCOLS", False);
	wm_delete_window = XInternAtom(XtDisplay(topLevel),
				"WM_DELETE_WINDOW", False);
	XtOverrideTranslations(topLevel,
		XtParseTranslationTable(
			"<Message>WM_PROTOCOLS: quit()"));
	XSetWMProtocols(XtDisplay(topLevel), XtWindow(topLevel),
			&wm_delete_window, 1);
	XtAppMainLoop(ac);
	return 0;
}
